module HAL where

import Conduit
import Data.Aeson
import Data.LanguageCodes (ISO639_1(..))
import Data.Text
import HAL.Client
import HAL.Doc.Corpus
import HAL.Doc.Struct
import HAL.Utils (langAbstractS)
import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)
import Protolude
import Servant.API
import Servant.Client (BaseUrl(..), Scheme(..), ClientM, ClientError, runClientM, mkClientEnv)

batchSize :: Int
batchSize = 1000

type Query = Text
type Start = Int
type Limit = Integer
type Count = Integer

getMetadataWith :: Query
                -> Maybe Start
                -> Maybe Limit
                -> Maybe ISO639_1
                -> IO (Either ClientError (Response Corpus))
getMetadataWith q start_ limit lang = do
  runHalAPIClient $ search (Just $ requestedFields lang) [q] Nothing start_ limit

getMetadataWithC :: Query
                 -> Maybe Start
                 -> Maybe Limit
                 -> Maybe ISO639_1
                 -> IO (Either ClientError (Maybe Count, ConduitT () Corpus IO ()))
getMetadataWithC q start_ limit lang = do
  -- First, estimate the total number of documents
  eCount <- countResults q
  pure $ get' <$> eCount
  where
    get' :: Count
         -> (Maybe Count, ConduitT () Corpus IO ())
    get' numFound_ =
      ( Just numResults
      , yieldMany [0..]
        .| takeC (fromInteger numPages)
        .| concatMapMC (getPage start'))
      where
        start' = fromMaybe 0 start_
        rows' = min numFound_ $ fromMaybe numFound_ limit
        numResults = rows' - (fromIntegral start')
        numPages = numResults `div` (fromIntegral batchSize) + 1

    getPage :: Start -> Int -> IO [Corpus]
    getPage start' pageNum = do
      let offset = start' + pageNum * batchSize
      eRes <- runHalAPIClient $ search (Just $ requestedFields lang) [q] Nothing (Just offset) (Just $ fromIntegral batchSize)
      pure $ case eRes of
        Left _ -> []
        Right (Response { _docs }) -> _docs
    -- printDoc :: Corpus -> IO Corpus
    -- printDoc c@(Corpus { .. }) = do
    --   putText $ show _corpus_title
    --   pure c

countResults :: Query -> IO (Either ClientError Count)
countResults q = do
  -- First, estimate the total number of documents
  eRes <- runHalAPIClient $ search (Just $ requestedFields Nothing) [q] Nothing (Just 0) (Just 1) :: IO (Either ClientError (Response Corpus))
  pure $ _numFound <$> eRes

requestedFields :: Maybe ISO639_1 -> Text
requestedFields (Just EN) = "docid,title_s,en_abstract_s,submittedDate_s,source_s,authFullName_s,authOrganism_s"
requestedFields (Just lang) = "docid,title_s,en_abstract_s," <> langAbstractS lang <> ",submittedDate_s,source_s,authFullName_s,authOrganism_s"
requestedFields _ = requestedFields (Just EN)

structFields :: Text
structFields = "docid,label_s,parentDocid_i"

runHalAPIClient :: (FromJSON doc, ToHttpApiData doc) =>
  ClientM (Response doc) -> IO (Either ClientError (Response doc))
runHalAPIClient cmd = do
  manager' <- newManager tlsManagerSettings
  runClientM cmd (mkClientEnv manager' $ BaseUrl Https "api.archives-ouvertes.fr" 443 "")

runStructureRequest :: Maybe Text -> IO (Either ClientError (Response Struct))
runStructureRequest rq =
  runHalAPIClient $ structure (Just structFields) rq (Just 10000)

runSearchRequest :: [Text] -> Maybe ISO639_1 -> IO (Either ClientError (Response Corpus))
runSearchRequest rq lang =
  runHalAPIClient $ search (Just $ requestedFields lang) rq Nothing Nothing Nothing

generateRequestByStructID :: Text -> [Text] -> Text
generateRequestByStructID rq struct_ids =
  "text:" <>
  rq
  <> " AND "
  <> "structId_i:("
  <> (flattenPipe struct_ids)
  <> ")"

flattenPipe :: [Text] -> Text
flattenPipe [] = ""
flattenPipe (x:[]) = x
flattenPipe (x:xs) = x <> " || " <> flattenPipe xs
